/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 2011,2012
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 

/*----------------------------------------------------------------------------
File Name:  ODPPCmnApiX.H

Purpose:    Provides the declarations for user written ODPP service providers,
            including the function and structure declarations. This file will
            be used by developers writing their own ODPP service providers
            
Functions:

   Name              Purpose

Structures:

   Name              Purpose

Author:
   Ritesh Tandon     IBM ISL 06/Mar/20012
----------------------------------------------------------------------------*/
#include "ODPPCmnAPI.h"

#ifndef __ODPPCMNAPI21_XH__
#define __ODPPCMNAPI21_XH__


/* Eye catcher for DP_FIELD_DEF_X */
#define DP_FIELD_DEF_X_EC           "FLDX"

/* Eye catcher for DP_INIT_OP_DEF_X */
#define DP_INIT_OP_DEF_X_EC         "OPRX"

/* Eye catcher for DP_SVC_DEF_X */
#define DP_SVC_DEF_X_EC             "SVCX"

/* Eye catcher for DP_PRV_DEF_X */
#define DP_PRV_DEF_X_EC             "PRVX"

/* Eye catcher for DP_BLK_DEF_X */
#define DP_BLK_DEF_X_EC             "BLKX"

/* Eye catcher for DP_LDS_FLD_DEF_X */
#define DP_LDS_FLD_DEF_X_EC         "LFLX"

/* Eye catcher for DP_LDS_ROW_DEF_X */
#define DP_LDS_ROW_DEF_X_EC         "LDRX"


//---------------------------------------------------------------
//  Name:          DP_FIELD_DEF_X
// 
//  Description:                                                 
// 
//  A Field Definition should describe all of the attributes of  
//  a field.  A field descriptor used for input will describe    
//  all of the attributes of that input as well as setting a     
//  pointer to that field.  A field descriptor used for output   
//  will set a pointer to where the data should be placed, and   
//  attributes should be set as to what form the data is in      
//  that the caller expects. Masked data can be returned either  
//  by modifying the Source or by the Target
// --------------------------------------------------------------
typedef struct DP_FIELD_DEF_X
{
   struct DP_FIELD_DEF_X
                 *pNext;               // Next element in chain of DP_FIELD_DEFINE
   char           cEyeCatcher[4];      // Eye catcher
   short          sStructVer;          // version of this structure 
   short          sStructLen;          // length of this structure 
   short          sDatatype;           // data type of the input data 
   int            iLength;             // column length in case of string data types
   int            iPrecision;          // precision of the input decimal data, 
                                       // and precesion of ORA_VARNUM
   short          sScale;              // scale of the input decimal data
   int            iCodePage;           // CodePage of data to be passed
   char           cDBMSType;           // DBMSType of data to be passed
   ODPP_WCHAR    *pColNameW;           // WideChar Column Name.
   char          *pColNameA;           // SBCS/MBCS Column Name.
   int            iColNameWBytes;      //Size of Wide Char Column Name buffer in bytes
   int            iColNameABytes;      //Size of SBCS/MBCS Column Name buffer in bytes
}DP_FIELD_DEF_X;

/*-------------------------------------------------------------*
 * Name:          DP_INIT_OP_DEF_X                             *
 *
 * Description:                                                *
 *
 * Init Operand Definition contains all the initialization     *
 * operands required for the current service                   *
 *------------------------------------------------------------*/
typedef struct DP_INIT_OP_DEF_X
{
   struct DP_INIT_OP_DEF_X
                 *pNext;               //Pointer to next element in chain of operands
   char           cEyeCatcher[4];      // Eyecatcher
   short          sStructVer;          // version of this structure 
   short          sStructLen;          // length of this structure 

   // Parameter ID
   unsigned short usParameterID;       // ODPP Parameter
   
   // Parameter names. Used in situations when there is no parameter ID 
   // usParameterID = 0 when a parameter name is passed
   ODPP_WCHAR    *pParamNameW;         // Wide Char Parameter Name
   char          *pParamNameA;         // SBCS/MBCS Parameter Name
   char           cPrmNameDBMSType;    // DBMS type of parameter name
   int            iPrmNameCodePage;    // Code page of parameter name
   int            iPrmNameBufBytes;    // Size of parameter name buffer in bytes.
   //Parameter values
   int            iValueSubType;       // Type of parameter value as defined below
                                       // any value except these are user defined values
                                       // and indicates the use of pUserVal.
                                       //================================================
                                       // Custom service providers can define their own 
                                       // values for their usage
                                       //================================================
   char           cValDBMSType;        // DBMS type of the parameter value
   int            iValCodePage;        // Code page of the parameter value
   int            iValABytes;          // Size of pValA buffer in bytes.
   int            iValWBytes;          // Size of pVal buffer in bytes.
   
   unsigned int   uiVal;               // numeric parameter value
   ODPP_WCHAR	  *pVal;                // wide-string param value
   char	        *pValA;               // SBCS/MBCS string param value
   void          *pUserVal;            // User defined parameter value 
}DP_INIT_OP_DEF_X;

//---------------------------------------------------------------------- 
// Name:          DP_SVC_DEF_X
// Description:                                                 
// The Service Definition contains the operands and field definition lists
// and the other service definition type initialization parameters.
//---------------------------------------------------------------------- 
typedef struct DP_SVC_DEF_X
{
   char           cEyeCatcher[4];      // Eye Catcher
   short          sStructVer;          // version of this structure
   short          sStructLen;          // length of this structure    
   char           bRefreshSVCDef;      // [For Future Use] When TRUE, this 
                                       // causes the existing the existing 
                                       // service definition to be updated
   char		      bCopyToDest;         // Copy the masked data to destination buf
   short	         sParamCount;  	      // Count of elements in pOperands
   short	         sFldCount;	         // Count of elements in pFldDef

   // Default Code Page and DBMS Type for Parameter names and values
   //
   // Note: Codepage/DBMSType values set for individual parameters/fields
   // take precedence over these value
   char           cCtrlDataDBMSType;   // DBMSType of control Data
                                       // in DP_INIT_OP_DEF_X and DP_FIELD_DEF_X
   int            iCtrlDataCP;         // Code page of control data 
                                       // in DP_INIT_OP_DEF_X and DP_FIELD_DEF_X

   DP_INIT_OP_DEF_X 
                 *pParams;             // Pointer to array or first element in
                                       // chain of DP_INIT_OP_DEF_X. Provides all 
                                       // needed operands.
   DP_FIELD_DEF_X
                 *pFldDef;             // Pointer to array or first element in
                                       // chain of DP_FIELD_DEF_X.

}DP_SVC_DEF_X;

//---------------------------------------------------------------------- 
// Name:          DP_SRCCOL_DEF_X                                 
// Description:                                                 
// Source column information. Will contain either the col index or name.
// If both or neither is provided, it will be an error
//---------------------------------------------------------------------- 
typedef struct DP_SRCCOL_DEF_X
{
   struct DP_SRCCOL_DEF_X
                 *pNext;               // Pointer to next element in chain of DP_SRCCOL_DEF_I  
   
   short          sColIndex;           // Index of the column containing source data.
   
   DP_FIELD_DEF_X
                 *pSrcFldDef;          // Pointer to User supplied Source Field Definition
}DP_SRCCOL_DEF_X;

//---------------------------------------------------------------------- 
// Name:          DP_BLK_DEF_X                                    
// Description:                                                 
// Control block for an individual provider initialization.
// Supplies all of the information a provider needs to function.
//---------------------------------------------------------------------- 
typedef struct DP_BLK_DEF_X
{
   char           cEyeCatcher[4];      // EyeCatcher
   short          sStructVer;          // Version of this structure
   short          sStructLen;          // Length of this structure    
   char           bCopyToDest;         // When TRUE, output is copied to dest buffer.
                                       // else output is copied to the source buffer

   short          sFlags;              // Flags controlling the behaviour of providers
                                       // specified for parameter ODPP_OPR_FLAGS
                                       // For a list of flags see ODPPCmnOprH.h
   
   short          sMethod;             //ODPP_METHOD_MASK/ODPP_METHOD_RANDOM/ODPP_METHOD_HASH
                                       
   short          sTraceLevel;         //Trace Levels for the service provider
                                       //Defined in ODPPCmnOprH.h
                                       //#define ODPP_TRACE_ERROR
                                       //#define ODPP_TRACE_WARNING
                                       //#define ODPP_TRACE_INFO

   short          sSrcColCount;        // Source column count

   short          sDiscardLimit;       // Total number of errors after which ODPP
                                       // error subsystem should fail the 
                                       // Provider_Service call

   char           cSysDBMSType;        // System specific DBMSType

   int            iSysCodePage;        // System specific CodePage. By default 
                                       // this is the OS default code page

   int            iSvcToken;           // Service Provider Service Token as 
                                       // assigned by the ODPP Service Manager

   DP_FIELD_DEF_X 
                 *pFldDef;             // Pointer to an array first element in 
                                       // the chain of DP_FIELD_DEF_X(s)

   void          *pExt;                // Placeholder to hold service provider 
                                       // specific control data. 
   // Allocated and Managed by the service provider. Service providers can use 
   // the above pointer pExt to preserve their instance specific data. This 
   // pointer lives till service provider is terminated. Cleaning this area 
   // will be responsibility of the individual Service Provider

   void          *pSrvMgrBlk;          // [Future Use] Service Manager specific block. 
                                       // This will be used to provide any ODPP specific 
                                       // functionality to service providers for e.g.
                                       // Data Conversion, Error Recording etc.

   DP_SRCCOL_DEF_X
                  SrcCol;              // Source column link list

}DP_BLK_DEF_X;


//---------------------------------------------------------------------- 
// Name:          DP_LDS_FLD_DEF_X                                
// Description:   
// Field definition representing an ODPPLDS field.       
// This would be used in an ODPPLDSGetData API call   
// to provide search and field(s) to the provider                                                            
//---------------------------------------------------------------------- 
typedef struct DP_LDS_FLD_DEF_X
{
   struct DP_LDS_FLD_DEF_X
                 *pNext;               // Pointer to next element in 
                                       // chain of DP_LDS_FLD_DEF_I
   char           cEyeCatcher[4];      // Eye catcher
   short          sStructVer;          // Version of this structure
   short          sStructLen;          // Length of this structure    
   char           bNull;               // Null indicator
   int            iBufLen;             // Max length of pBuf in Bytes
   int            iErrorCode;          // Error code for this data
   int            iSrcBufBytes;        // This tells the number of C buffer bytes
                                       // required to hold the source data.
   int            iLdsBufBytes;        // This tells the number of C buffer bytes
                                       // required to hold the LDS data.
   int            iColPos;             // Ordinal position of Column in lookup table

   int            iColPreserveOpt;     // Preserve Options for replacement columnns

   DP_FIELD_DEF_X
                 *pFldDefSrc;          // Field definition sent by the caller

   DP_FIELD_DEF_X
                 *pFldDefLDS;          // Field definition retrieved from the 
                                       // LDS library
   unsigned char  *pBuf;               // buffer for field data
}DP_LDS_FLD_DEF_X;


//---------------------------------------------------------------------- 
// Name:          DP_LDS_ROW_DEF_X                                
// Description:   
// LDS Row definition representing a list of ODPPLDS fields.
// Used in the ODPPLDSGetDataMultiple API call to fetch multiple
// rows similar to ...GetData
//---------------------------------------------------------------------- 
typedef struct DP_LDS_ROW_DEF_X
{
   struct DP_LDS_ROW_DEF_X
                 *pNext;               // Pointer to next element in chain of DP_LDS_ROW_DEF_I
   char           cEyeCatcher[4];      // Eye catcher
   short          sStructVer;          // Version of this structure
   short          sStructLen;          // Length of this structure    
   short          sCount;              // Count of DP_LDS_FLD_DEF_X(s) 
                                       // in form of a chain of elements
   RETVAL         iErrCode;            // Error code for this row
   DP_LDS_FLD_DEF_X
                 *pLdsFld;             // Pointer to the first element in the 
                                       // chain of DP_LDS_FLD_DEF_X(s).                 
}DP_LDS_ROW_DEF_X;

#ifdef _WIN32
   /* These are individual service provider interface APIs declares */
   typedef RETVAL (__stdcall *PINITX)     (
                                                DP_BLK_DEF_X *pCtrlBlk, 
                                                DP_SVC_DEF_X *pSvcDef
                                          );

   typedef RETVAL (__stdcall *PSERVICEX)  (
                                                short sMethod, 
                                                DP_BLK_DEF_X *pCtrlBlk, 
                                                DP_ROWSET_DEF *pRowSet
                                          );

   typedef RETVAL (__stdcall *PTERMINATEX)(
                                                DP_BLK_DEF_X *pCtrlBlk
                                          );
   
   typedef RETVAL (__stdcall *PGETERRORMSGX)(
                                                RETVAL ErrorCode,
                                                int iLanguage,
                                                ODPP_WCHAR *pErrMsg
                                           );

   typedef RETVAL (__stdcall *PGETINFOX)   (
                                                DP_BLK_DEF_X *pCtrlBlk, 
                                                short sRequest,  
                                                void *ptr,
                                                int *pBufLen
                                          );

   /*These are LDS interface APIs declares */
   typedef RETVAL (__stdcall *PENVINITX)   (void);
   
   typedef RETVAL (__stdcall *PCONNECTX)   (
                                                DP_BLK_DEF_X *pCtrlBlk,
                                                int *pLdsToken, 
                                                DP_INIT_OP_DEF_X *pOperands,
                                                short sOprCount
                                          );
   
   typedef RETVAL (__stdcall *PDISCONNECTX)(
                                                int iLdsToken
                                          );
   
   typedef RETVAL (__stdcall *PGETCOLINFOX)(
                                                int iLdsToken,
                                                DP_LDS_FLD_DEF_X *pLdsColDefs,
                                                short sFldCount,
                                                DP_INIT_OP_DEF_X *pOperands,
                                                short sOprCount
                                          );
   
   typedef RETVAL (__stdcall *PGETDATAX)   (
                                                int iLdsToken,
                                                DP_LDS_FLD_DEF_X *pSearchCols,
                                                DP_LDS_ROW_DEF_X *pDstRows,
                                                DP_INIT_OP_DEF_X *pOperands,
                                                short sOprCount,
                                                int iMaxRows,
                                                char bFetchMultiple,
                                                int *pRowsFetched
                                          );

   typedef RETVAL (__stdcall *PCLOSEFETCHX)(
                                                int iLdsToken
                                          );

   typedef RETVAL (__stdcall *PGETLDSINFOX)(
                                                int iLdsToken,
                                                DP_INIT_OP_DEF_X *pLkpOperands,
                                                short sOprCount,
                                                void *ptr
                                          );
   
   typedef RETVAL (__stdcall *PENVTERMX)  (void);

#else
   /* These are individual service provider interface APIs declares */
   typedef RETVAL ( *PINITX)              (
                                                DP_BLK_DEF_X *pCtrlBlk, 
                                                DP_SVC_DEF_X *pSvcDef
                                          );

   typedef RETVAL ( *PSERVICEX)           (
                                                short sMethod, 
                                                DP_BLK_DEF_X *pCtrlBlk, 
                                                DP_ROWSET_DEF *pRowSet
                                          );

   typedef RETVAL ( *PTERMINATEX)         (
                                                DP_BLK_DEF_X *pCtrlBlk
                                          );
   
   typedef RETVAL ( *PGETERRORMSGX)       (
                                                RETVAL ErrorCode,
                                                int iLanguage,
                                                ODPP_WCHAR *pErrMsg
                                           );

   typedef RETVAL ( *PGETINFOX)           (
                                                DP_BLK_DEF_X *pCtrlBlk, 
                                                short sRequest,  
                                                void *ptr,
                                                int *pBufLen
                                          );

   /* These are LDS interface APIs declares */
   typedef RETVAL ( *PENVINITX)           (void);
   
   typedef RETVAL ( *PCONNECTX)           (
                                                DP_BLK_DEF_X *pCtrlBlk,
                                                int *pLdsToken, 
                                                DP_INIT_OP_DEF_X *pOperands,
                                                short sOprCount
                                          );
   
   typedef RETVAL ( *PDISCONNECTX)        (
                                                int iLdsToken
                                          );
   
   typedef RETVAL ( *PGETCOLINFOX)        (
                                                int iLdsToken,
                                                DP_LDS_FLD_DEF_X *pLdsColDefs,
                                                short sFldCount,
                                                DP_INIT_OP_DEF_X *pOperands,
                                                short sOprCount
                                          );
   
   typedef RETVAL ( *PGETDATAX)           (
                                                int iLdsToken,
                                                DP_LDS_FLD_DEF_X *pSearchCols,
                                                DP_LDS_ROW_DEF_X *pDstRows,
                                                DP_INIT_OP_DEF_X *pOperands,
                                                short sOprCount,
                                                int iMaxRows,
                                                char bFetchMultiple,
                                                int *pRowsFetched
                                          );

   typedef RETVAL ( *PCLOSEFETCHX)        (
                                                int iLdsToken
                                          );

   typedef RETVAL ( *PGETLDSINFOX)        (
                                                int iLdsToken,
                                                DP_INIT_OP_DEF_X *pLkpOperands,
                                                short sOprCount,
                                                void *ptr
                                          );
   
   typedef RETVAL ( *PENVTERMX)           (void);

#endif //#ifdef _WIN32

//---------------------------------------------------------------------- 
// Name:          DP_PRVAPI_DEF_X                                 
// Description:   Holds the service providers api pointers                                    
//---------------------------------------------------------------------- 
typedef struct DP_PRVAPI_DEF_X
{
   PINITX         pInit;               // Init API pointer
   PSERVICEX      pService;            // Service API pointer
   PTERMINATEX    pTerm;               // Terminate API pointer
   PGETERRORMSGX  pGetErrMsg;          // GetErrorMessage API pointer
   PGETINFOX      pGetInfo;            // GetInfo API pointer
}DP_PRVAPI_DEF_X;

#ifdef _DECLARE_SRVPRV_INTERFACE
   RETVAL Init       (
                        DP_BLK_DEF_X *pCtrlBlk, 
                        DP_SVC_DEF_X *pSvcDef
                     );

   RETVAL Service    (
                        short sMethod, 
                        DP_BLK_DEF_X *pCtrlBlk, 
                        DP_ROWSET_DEF *pRowSet
                     );

   RETVAL Terminate  (
                        DP_BLK_DEF_X *pCtrlBlk
                     );
   
   RETVAL GetErrorMsg(
                        RETVAL ErrorCode,
                        int iLanguage,
                        ODPP_WCHAR *pErrMsg
                     );

   RETVAL GetInfo    (
                        DP_BLK_DEF_X *pCtrlBlk, 
                        short sRequest,  
                        void *ptr,
                        int *pBufLen
                     );
#endif //_DECLARE_SRVPRV_INTERFACE

/* These are LDS interface APIs declares */
#ifdef _DECLARE_LDS_INTERFACE
   RETVAL ODPPLDSEnvInit(void);
      
   RETVAL ODPPLDSConnect(
                           DP_BLK_DEF_X *pCtrlBlk,
                           int *pLdsToken, 
                           DP_INIT_OP_DEF_X *pOperands,
                           short sOprCount
                        );

   RETVAL ODPPLDSDisconnect(
                              int iLdsToken
                           );
      
   RETVAL ODPPLDSGetData(
                              int iLdsToken,
                              DP_LDS_FLD_DEF_X *pSearchCols,
                              DP_LDS_ROW_DEF_X *pDstRows,
                              DP_INIT_OP_DEF_X *pOperands,
                              short sOprCount,
                              int iMaxRows,
                              char bFetchMultiple,
                              int *pRowsFetched
                        );
      
   RETVAL ODPPLDSCloseFetch(
                              int iLdsToken
                           );

   RETVAL ODPPLDSGetColInfo(
                              int iLdsToken,
                              DP_LDS_FLD_DEF_X *pLdsColDefs,
                              short sFldCount,
                              DP_INIT_OP_DEF_X *pOperands,
                              short sOprCount                           
                           );

   RETVAL ODPPLDSGetInfo(
                           int iLdsToken,
                           DP_INIT_OP_DEF_X *pLkpOperands,
                           short sOprCount,
                           void *ptr
                        );
      
   RETVAL ODPPLDSEnvTerm(void);
#endif //_DECLARE_LDS_INTERFACE

#endif //__ODPPCMNAPI21_XH__
